'use strict';

const assert = require('assert');

async function doTourneyPlacement({
  playerId,
  game_submode,
  tier,
  db,
}) {
  if (tier > 5) {
    tier = 5;
  }

  // doing new tourney placement (join or create) - look for an existing tourney to add the player to
  const now = Date.now();
  let new_tourney_id;
  let tourney_instance;
  const updated_tourney_res = await db
    .collection('script.inn_tournaments')
    .findOneAndUpdate(
      {
        game_submode,
        'players.99': { $exists: false }, // no more than 100 active players
        tier,
      },
      {
        $push: { players: { _id: playerId } },
      },
      {
        returnDocument: 'after',
      },
    );
  if (updated_tourney_res) {
    tourney_instance = updated_tourney_res;
  } else {
    // didn't add player to any active tourney, so create one
    tourney_instance = {
      game_submode,
      players: [{ _id: playerId }],
      started_at: now,
      tier,
    };
    const { insertedId } = await db
      .collection('script.inn_tournaments')
      .insertOne(tourney_instance);
    assert.ok(insertedId);
    new_tourney_id = insertedId;
  }
  // still doing new tourney placement - update calling player's player_inn_state doc
  const player_inn_submode_state = {
    active_tourney_id: new_tourney_id || tourney_instance._id,
    challenges_left: 10,
    current_rank: tourney_instance.players.length,
    current_tier: tourney_instance.tier,
    last_num_peanuts: 0,
    last_rank_change_at: now,
    tourney_entered_at: now,
  };
  const inn_state_res = await db
    .collection('script.player_inn_state')
    .updateOne(
      { _id: playerId },
      {
        $set: {
          [`player_inn_state.${game_submode}`]: player_inn_submode_state,
        },
      },
    );
  assert.ok(inn_state_res.modifiedCount);

  return {
    player_inn_submode_state,
    tourney_instance,
  };
}
module.exports = doTourneyPlacement;
