const ItemGenerator = require('../../../game_items/ItemGenerator.js');
const LightHelmet = require('../../../game_items/equipment/helmet/LightHelmet.js');
const LightTorsoe = require('../../../game_items/equipment/torsoe/LightTorsoe.js');
const LightBoots = require('../../../game_items/equipment/boots/LightBoots.js');
const Trinket = require('../../../game_items/equipment/off_hand/Trinket.js');
const getMobEquipmenAssignmentFromBags = require('./getMobEquipmenAssignmentFromBags');

const goblin_elementalist = (level) => {
  const handle = 'goblin_elementalist';
  const AI_PRIORITIES = ['dps_ranged', 'interference', 'dps_close', 'support', 'tank'];
  const bags = [{
    items: [
      ItemGenerator.getRandomItem(LightHelmet, 0, Math.min(1,0.1+level/100)),
      ItemGenerator.getRandomItem(LightTorsoe, 0, Math.min(1,0.1+level/100)),
      ItemGenerator.getRandomItem(LightBoots, 0, Math.min(1,0.1+level/100)),
      ItemGenerator.getRandomItem(Trinket, 0, Math.min(1,0.1+level/100)),
    ],
  }];

  return {
    handle,
    level,
    getAbilityTrees: ((bound_level) => ([{
      handle: 'goblin_elementalist_abilities',
      abilities: {
        move: { handle: 'move', level: 1 },
        melee_attack: { handle: 'melee_attack', level: 1 },
        burning_hands: { handle: 'burning_hands', level: Math.min(10,Math.round(1+bound_level/2)) },
        lightning: { handle: 'lightning', level: Math.min(10,Math.round(1+bound_level/2)) },
        mana_node: { handle: 'mana_node', level: Math.min(10,Math.round(1+bound_level/2)) },
        ice_nine: { handle: 'ice_nine', level: Math.min(10,Math.round(1+bound_level/2)) },
        molotov_cocktail: { handle: 'molotov_cocktail', level: Math.min(10,Math.round(1+bound_level/2)) },
      },
    }])).bind(null, level),
    getBaseAttributes: ((bound_level) => ({
      learning: Math.round(2.9 + bound_level*0.9 + Math.random()*3),
      perception: Math.round(1.6 + bound_level*0.4 + Math.random()*3),
      mana: Math.round(2.5 + bound_level*0.65 + Math.random()*3),
      toughness: Math.round(1.8 + bound_level*0.5 + Math.random()*3),
      strength: Math.round(1.2 + bound_level*0.35 + Math.random()*3),
      willpower: Math.round(2.7 + bound_level*0.65 + Math.random()*3),
      dexterity: Math.round(2.3 + bound_level*0.75 + Math.random()*3),
    })).bind(null, level),
    getDefaultAIConfig: () => AI_PRIORITIES,
    xpValue: level * 900,
    bags,
    build: {
      ai: {
        priorities: AI_PRIORITIES,
        settings: {
          chanceToSkipRoles: true,
          autoMove: true,
          waitForAPMP: false,
        },
      },
      ai_unit: true,
      beast: false,
      equipment_assignment: getMobEquipmenAssignmentFromBags(bags),
      equipped_abilities: ['move','melee_attack','burning_hands','lightning','mana_node','ice_nine','molotov_cocktail'],
      handle,
      humanoid: true,
      level,
      team: 'black',
      undead: false,
      unit: true,
    },
  };
};
module.exports = goblin_elementalist;
